local MAJOR = "StarLibPluginResourceTools-1.0" 
local MINOR = 9

local PluginResourceTools = LibStub:NewLibrary(MAJOR, MINOR)
if not PluginResourceTools then return end

if not PluginResourceTools.__index then
	PluginResourceTools.__index = PluginResourceTools
end

local scriptProfile = GetCVar("scriptProfile") == "1"
local oldtotalcpu, oldtotalmem
local lastcputime, lastmemtime = GetTime(), GetTime()
local oldcpu = {}
local oldmem = {}
local oldmem = {}
local oldCPUTable = {}


-- @name PluginResourceTools:New
-- @usage PluginResourceTools:New(visitor, environment, name, config) -- 
-- @parma environment This will be the environment when loadstring is callled.
-- @return A new StarPluginResourceTools object
function PluginResourceTools:New(environment)
	
	local obj = {}

	setmetatable(obj, self)
	
	environment.GetCPUUsage = self.GetCPUUsage
	environment.GetMemUsage = self.GetMemUsage
	environment.GetLatency = self.GetLatency
	environment.GetFramerate = GetFramerate
	environment.scriptProfile = scriptProfile
	
	return obj	
	
end

-- @name UpdateCPU
-- @usage PluginResourceTools.UpdateCPU()
-- @return NOthing
function PluginResourceTools.UpdateCPU()
	UpdateAddOnCPUUsage()
end

-- @name GetCPUUsage
-- @usage PluginResourceTools.GetCPUUsage(addon)
-- @param addon We'll get the cpu usage for this addon.
-- @return cpu, percent, cpudiff, totalCPU, totaldiff
function PluginResourceTools.GetCPUUsage(addon)
	if not scriptProfile then return 0, 0, 0, 0, 0 end
		
	if not oldcpu[addon] then oldcpu[addon] = {} end
	
	
	local totalCPU = 0.0001
	
	for i=1, GetNumAddOns() do
		totalCPU = totalCPU + (GetAddOnCPUUsage(i) )
	end	
	
	local cpu = GetAddOnCPUUsage(addon)

	local perc = cpu / totalCPU * 100
	local cpudiff = oldcpu[addon][1] and (cpu - oldcpu[addon][1]) or 0
	local totaldiff = oldtotalcpu and (totalCPU - oldtotalcpu) or 0

	oldtotalcpu = totalCPU
		
	oldcpu[addon][1] = cpu
	oldcpu[addon][2] = perc
	oldcpu[addon][3] = cpudiff
	oldcpu[addon][4] = totalCPU
	oldcpu[addon][5] = totaldiff
	
	return unpack(oldcpu[addon])
end

-- @name UpdateMem
-- @usage PluginResourceTools.Update()
-- @return Nothing
function PluginResourceTools.UpdateMem() 
	UpdateAddOnMemoryUsage()
end

-- @name GetMemUsage
-- @usage PluginResourceTools.GetMemUsage(addon)
-- @param addon We'll get the mem usage for this addon.
-- @return mem, percent, memdiff, totalMem, totaldiff
function PluginResourceTools.GetMemUsage(addon)
	
	if not oldmem[addon] then oldmem[addon] = {} end
	
	local totalMem = 0.0001
	
	for i=1, GetNumAddOns() do
		totalMem = totalMem + (GetAddOnMemoryUsage(i) )
	end	
	
	local mem = GetAddOnMemoryUsage(addon)
	local perc = mem / totalMem * 100
	local memdiff = oldmem[addon][1] and (mem - oldmem[addon][1]) or 0
	local totaldiff = oldtotalmem and (totalMem - oldtotalmem) or 0
	
	oldtotalmem = totalMem
		
	oldmem[addon][1] = mem
	oldmem[addon][2] = perc
	oldmem[addon][3] = memdiff
	oldmem[addon][4] = totalMem
	oldmem[addon][5] = totaldiff
	
	return mem, perc, memdiff, totalMem, totaldiff
end


local lastlat = 0
function PluginResourceTools.GetLatency()
	local _, _, latency = GetNetStats()
	if type(latency) == "number" then lastlat = latency else latency = lastlat end
	return latency
end

function PluginResourceTools.GetFramerate()
	return GetFramerate() or 0
end
